#ifndef _ALGORITHMS_INTERFACE
#define _ALGORITHMS_INTERFACE

#include <stdio.h>
#include <vector>
#include "algorithms/trail_division_algorithm.hpp"

std::vector<std::string> cpp_trial_division(char * n);

#endif
